@@ -60,7 +60,7 @@ blacklist_check(unsigned int fd)
       logmessage("DEBUG: fd: %d bypassing other modules\n", fd);
     }
 
-    logmessage("rcpt=%lu, type=blacklist, host=%s (%s), from=%s, to=%s, size=%s\n",
+    logmessage("rcpt=%lu, blacklist=block, host=%s (%s), from=%s, to=%s, size=%s\n",
       rcpt_count,                       /* recipient count      */
       host_array[fd][2],                /* host address         */
       host_array[fd][0],                /* hostname             */
@@ -110,7 +110,7 @@ blacklist_sender_check (unsigned int fd)
       logmessage("DEBUG: fd: %d bypassing other modules\n", fd);
     }
 
-    logmessage("rcpt=%lu, type=blacklist_sender, host=%s (%s), from=%s, to=%s, size=%s\n",
+    logmessage("rcpt=%lu, blacklist_sender=block, host=%s (%s), from=%s, to=%s, size=%s\n",
       rcpt_count,               /* recipient count      */
       host_array[fd][2],        /* ip address           */
       host_array[fd][0],        /* hostname             */
@@ -164,7 +164,7 @@ blacklist_dnsname_check (unsigned int fd)
       logmessage("DEBUG: fd: %d bypassing other modules\n", fd);
     }
 
-    logmessage("rcpt=%lu, type=blacklist_dnsname, host=%s (%s), from=%s, to=%s, size=%s\n",
+    logmessage("rcpt=%lu, blacklist_dnsname=block, host=%s (%s), from=%s, to=%s, size=%s\n",
       rcpt_count,               /* recipient count      */
       host_array[fd][2],        /* ip address           */
       host_array[fd][0],        /* hostname             */
