@@ -37,8 +37,21 @@
 int
 w_mysql_query(unsigned int volatile fd, const char *function)
 {
-  /* increase mysql query counter */
-  mysql_count++;
+
+  /* cut down latency, do not do any queries for 30 seconds if 3 failures occur */
+  if(mysql_failure_count >= 3)
+  {
+
+    /* has 30 seconds passed? */
+    if(timenow >= (last_mysql_failure+30))
+    {                                 /* timer has expired     */
+      mysql_failure_count=0;
+      mysql = db_connect(MYSQLDBASE); /* reconnect to database */
+    } else {                          /* timer has not expired */
+
+      return (-1);                    /* so we fail the query  */
+    }
+  }
 
   /* catch query timeouts */
   if (sigsetjmp (sjmp, 1))
@@ -50,6 +63,12 @@ w_mysql_query(unsigned int volatile fd, const char *function)
     mysql_failure_count++;
     last_mysql_failure=timenow;
 
+    /* reset the timer */
+    alarm (0);
+
+    /* unset the signal handler */
+    signal (SIGALRM, SIG_DFL);
+
     mysql_array[fd][0] = -1;
     return (-1); /* failure */
   }
@@ -60,29 +79,17 @@ w_mysql_query(unsigned int volatile fd, const char *function)
   /* mysql timeout */
   alarm(mysql_timeout);
   
-  /* cut down latency, do not do any queries for 5 minutes if 3 failures occur */
-  if(mysql_failure_count >= 3)
-  {
-    /* has 30 seconds passed? */
-    if(timenow >= (last_mysql_failure+30))
-    {
-                           /* timer has expired */
-      mysql_failure_count=0;
-                           /* reconnect to database */
-      mysql_close(mysql);
-      mysql = db_connect(MYSQLDBASE);
-
-    } else {               /* timer has not expired */
-
-      /* reset the timer & unset the signal handler */
-      alarm (0);     signal (SIGALRM, SIG_DFL);
-      return (-1);
-    }
-  }
-    
   /* fire off query */
   if (mysql_query(mysql, mysqlquery_array[fd]) != 0)
+  {
+    /* reset the timer */
+    alarm (0);
+
+    /* unset the signal handler */
+    signal (SIGALRM, SIG_DFL);
+
     return (-1);
+  }
   
   /* reset the timer */
   alarm (0);
@@ -110,7 +117,7 @@ db_doquery(unsigned int volatile fd)
 {
   
   if(DEBUG > 1)
-    logmessage("DEBUG: fd: %d query=%ld, db_doquery(): %s\n", fd, mysql_count, mysqlquery_array[fd]);
+    logmessage("DEBUG: fd: %d db_doquery(): %s\n", fd, mysqlquery_array[fd]);
 
   /* fire off query */
   if (w_mysql_query(fd, "db_doquery") != 0)
@@ -173,7 +180,7 @@ db_charquery(unsigned int volatile fd)
 {      
 
   if(DEBUG > 1)
-    logmessage("DEBUG: fd: %d query=%ld, db_charquery(): %s\n", fd, mysql_count, mysqlquery_array[fd]);
+    logmessage("DEBUG: fd: %d db_charquery(): %s\n", fd, mysqlquery_array[fd]);
 
   /* fire off query */
   if (w_mysql_query(fd, "db_charquery") != 0)
@@ -201,7 +208,7 @@ db_charquery(unsigned int volatile fd)
         if(DEBUG > 1)
           logmessage("DEBUG: fd: %d row: %d data: %s (recieved)\n", fd, i, (char *)*row);
 
-        strncpy(mysqlchar_array[fd][i], (char*)*row, 256);
+        strncpy(mysqlchar_array[fd][i], (char*)*row, 64);
 
         if(DEBUG > 1)
           logmessage("DEBUG: fd: %d row: %d data: %s (extracted)\n", fd, i, mysqlchar_array[fd][i]);
@@ -235,7 +242,7 @@ db_optquery(unsigned int volatile fd)
 {
   
   if(DEBUG > 1)
-    logmessage("DEBUG: fd: %d query=%ld, db_optquery(): %s\n", fd, mysql_count, mysqlquery_array[fd]);
+    logmessage("DEBUG: fd: %d, db_optquery(): %s\n", fd, mysqlquery_array[fd]);
 
   /* fire off query */
   if (w_mysql_query(fd, "db_optquery") != 0)
@@ -295,7 +302,7 @@ db_deletequery(unsigned int volatile fd)
 start:
   
   if(DEBUG > 1)
-    logmessage("DEBUG: fd: %d query=%ld, db_deletequery(): %s\n", fd, mysql_count, mysqlquery_array[fd]);
+    logmessage("DEBUG: fd: %d, db_deletequery(): %s\n", fd, mysqlquery_array[fd]);
 
   /* fire off query */
   if (w_mysql_query(fd, "db_deletequery") != 0)
@@ -360,7 +367,7 @@ int
 db_printquery(unsigned int volatile fd)
 {
   if(DEBUG > 1)
-    logmessage("DEBUG: fd: %d query=%ld, db_printquery(): %s\n", fd, mysql_count, mysqlquery_array[fd]);
+    logmessage("DEBUG: fd: %d, db_printquery(): %s\n", fd, mysqlquery_array[fd]);
 
   /* fire off query */
   if (w_mysql_query(fd, "db_printquery") != 0)
@@ -459,7 +466,8 @@ MYSQL *db_connect(const char *dbname)
   if(!mysql_real_connect(db, MYSQLHOST, MYSQLUSER, MYSQLPASS, dbname, MYSQLPORT, NULL, 0))
   {
     logmessage("mysql_real_connect(): %s\n", mysql_error(db));
-    logmessage("NOT connected..\n");
+    mysql_failure_count++;
+    last_mysql_failure=timenow;
   } else {
     logmessage("connected..\n");
   }
