@@ -50,10 +50,16 @@
 #include <stdio.h>
 #include <mysql.h>
 #include <setjmp.h>
+/* Do not generate SIGPIPE */
+#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined(__NetBSD__)
+  #ifndef MSG_NOSIGNAL
+    #define MSG_NOSIGNAL SO_NOSIGPIPE
+  #endif
+#endif
 
 /* CONFIGS */
 #define PROJECT         "policyd"
-#define VERSION         "v1.78"
+#define VERSION         "v1.80"
 
 /* Miscellaneous constants */
 #define LISTENQ         1023    /* 2nd argument to listen() */
@@ -136,6 +142,10 @@ char *MYSQLOPT;
  int SENDERQUOTALIMIT;
  int SENDERMSGSIZE;
  int SENDER_INACTIVE_EXPIRE;
+ int SENDER_THROTTLE_AUTOBLACKLIST;
+ int SENDER_THROTTLE_AUTOBLACKLIST_NUMBER;
+ int SENDER_THROTTLE_AUTOBLACKLIST_EXPIRE;
+
  int RECIPIENTTHROTTLE;
  int RECIPIENTMSGLIMIT;
  int RECIPIENTTIMELIMIT;
@@ -160,7 +170,6 @@ char *MYSQLOPT;
 /* GLOBAL VARIABLES/ARRAYS */
  MYSQL * volatile mysql;
 unsigned long int rcpt_count;           /* total mails processed */
-unsigned long int mysql_count;          /* total mysql queries   */
 unsigned long int mysql_failure_count;  /* total mysql queries   */
 unsigned long int last_mail_time;       /* seconds since epoch   */
 unsigned long int last_mysql_failure;   /* seconds since epoch   */
@@ -171,10 +180,12 @@ unsigned long int mysql_timeout;        /* mysql query timeout   */
   int ssock;                            /* slave server socket   */
   int         mysql_array[MAXFDS][10];
   int      mysql_optarray[MAXFDS][1];
-  char    mysqlchar_array[MAXFDS][10][64];
-  char         host_array[MAXFDS][10][64];
-  char       policy_array[MAXFDS][10][64];
-  char      triplet_array[MAXFDS][10][64];
+  
+  char    mysqlchar_array[MAXFDS][20][64];
+  char         host_array[MAXFDS][20][64];
+  char       policy_array[MAXFDS][20][64];
+  char      triplet_array[MAXFDS][20][64];
+
   char   mysqlquery_array[MAXFDS][512];
   char    xgreylist_array[MAXFDS][128];
   char      extract_array[MAXFDS][64];
@@ -182,6 +193,7 @@ unsigned long int mysql_timeout;        /* mysql query timeout   */
   char  extract_host_addr[MAXFDS][64];
   char        return_code[MAXFDS][64];
   char extract_array_conf[MAXFDS][64];
+  
   unsigned int i[MAXFDS], instance_inc[MAXFDS], t;
   unsigned int tcount[MAXFDS], tquota[MAXFDS], trcpt[MAXFDS];
   char      tattrib_array[MAXFDS][1];
@@ -192,6 +204,8 @@ unsigned long int mysql_timeout;        /* mysql query timeout   */
   int  action_array[MAXFDS];
   char confbuf[256];
   char buf[MAXFDS][MAXLINE];
+  unsigned int buf_size[MAXFDS];
+  unsigned int buf_counter[MAXFDS];
 
 
 /*  PROTOTYPES */
@@ -200,14 +214,15 @@ unsigned long int mysql_timeout;        /* mysql query timeout   */
   int w_select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout);
   int w_socket(int family, int type, int protocol);
   int w_accept(unsigned int fd, struct sockaddr *sa, socklen_t *salenptr);
-  int w_write(unsigned int fd, void *ptr, int volatile nbytes);
+   void buf_write(unsigned int fd, const char *ptr, size_t nbytes);
   int socktimeout(unsigned int fd, unsigned int sec);
   int daemonize(int nochdir, int noclose);
   int w_tcp_conn_acl (const char *host);
-  int    cidr_ip_match (unsigned long ip, unsigned char *range);
+  int    cidr_ip_match (unsigned long ip, char *range);
   pid_t w_fork(void);
 const char *w_inet_ntop(int family, const void *addrptr, char *strptr, size_t len);
- ssize_t w_read(unsigned int volatile fd, void *ptr, size_t nbytes);
+ ssize_t w_read(unsigned int fd, char *ptr);
+ ssize_t w_write(unsigned int fd, const void *vbuf);
  ssize_t f_write(unsigned int volatile fd, const void *vptr, size_t n);
  void w_close(unsigned int fd);
  void w_bind(unsigned int fd, const struct sockaddr *sa, socklen_t salen);
@@ -215,7 +230,7 @@ const char *w_inet_ntop(int family, const void *addrptr, char *strptr, size_t le
  void sigalrm_handler (void);
 
   // functions
-  int chk_pol(unsigned int fd);
+  void chk_pol(unsigned int fd);
   int bindsock(unsigned int port, unsigned int qlen);
   int greylist_check(unsigned int fd);
   int spamtrap_check(unsigned int fd);
@@ -234,7 +249,7 @@ const char *w_inet_ntop(int family, const void *addrptr, char *strptr, size_t le
   int whitelist_dnsname_check(unsigned int fd);
   int blacklist_sender_check(unsigned int fd);
   int blacklist_dnsname_check(unsigned int fd);
-  int policy_reply(unsigned int fd, int code, int status);
+  void policy_reply(unsigned int fd, int code, int status);
   int blacklist_check(unsigned int fd);
   int extract_seconds(char *token);
   int parse_syslog_priority (char *str);
