@@ -38,6 +38,17 @@ w_select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct
   int n;
   if ((n=select(nfds, readfds, writefds, exceptfds, timeout)) < 0)
   {
+    if (errno == EINTR)
+    {
+      logmessage("warning: select(): system interupt\n");
+      return 0;
+    }
+    if (errno == EBADF)
+    {
+      logmessage("warning: select(): descriptor was not open\n");
+      return 0;
+    }
+
     logmessage("fatal: select(): %s\n", strerror(errno));
     exit(-1);
   }
@@ -136,123 +147,139 @@ w_listen(unsigned int fd, unsigned int backlog)
  *   return: number bytes read
  */ 
 ssize_t
-w_read(unsigned int volatile fd, void *ptr, size_t nbytes)
+w_read(unsigned int fd, char *ptr)
 {       
-  ssize_t         n;
+  ssize_t  n;
+  size_t   data_read = 0;                                    /* for debug only */
 
-  /* catch read timeouts */
-  if (sigsetjmp (sjmp, 1))
-  { 
-    logmessage("warning: read(): read timeout on fd: %d\n", fd);
-    return (0); /* close connection */
-  }
-
-  /* set the SIGALRM handler */
-  signal (SIGALRM, (void *) sigalrm_handler);
-  
-  /* read timeout */
-  alarm(3);
-  
-  /* read from a file descriptor */
-  if ((n = read(fd, ptr, nbytes)) == -1)
+  /* receive data. disable signals are do not wait */
+  while ((n = recv(fd, (void *) ptr + buf_counter[fd], 1, MSG_DONTWAIT | MSG_NOSIGNAL)) == 1)
   {
-    /* reset the timer & unset the signal handler */
-    alarm (0);           signal (SIGALRM, SIG_DFL);
+    data_read++;
+    buf_counter[fd]++;
+    buf_size[fd]++;
 
-    logmessage("warning: read(): %s on fd: %d\n", strerror(errno), fd);
-    return (0);                               /* close connection */
+    /* need at least 2 bytes to check against */
+    if (buf_counter[fd] > 2)
+    {
+      /* received policy protocol terminator */
+      if (strncmp(ptr + buf_counter[fd] - 2, "\n\n", 2) == 0)
+      {
+	if (DEBUG > 2)
+          logmessage("DEBUG: fd %d: %s: returning -2 after reading %d bytes\n",fd, __FUNCTION__, data_read);
+	
+        return -2;
+      }	
+    }
   }
 
-  if(DEBUG > 2)
+  /* check if recv returned an error */ 
+  if (n == -1)
   {
-    if(n > 0)
-      logmessage("DEBUG: fd: %d read(%d) bytes\n", fd, n);
-    else
-      logmessage("DEBUG: fd: %d connection closed\n", fd);
+    /* Ignore EAGAIN */
+    if (errno == EAGAIN) 
+    {
+      if (DEBUG > 2)
+        logmessage("DEBUG: fd: %d returning data_read = %d after EAGAIN\n", fd, data_read);
+      
+      return data_read;
+    }
+    
+    /* here something bad happened */
+    if (DEBUG > 2)
+      logmessage("DEBUG: fd: %d connection problem during read: %s\n", fd, strerror(errno));
+
+    return -1;
   }
 
-  /* reset the timer & unset the signal handler */
-  alarm (0);           signal (SIGALRM, SIG_DFL);
+  /* EOF was reached if n == 0 */
+  if (n == 0)
+  {
+    if(DEBUG > 2)
+      logmessage("DEBUG: fd: %d connection got an EOF, data_read = %d\n", fd, data_read);
+     
+    return -1;
+  }
 
-  return (n);                                 /* bytes read */
+  if (DEBUG > 2)  
+    logmessage("DEBUG: fd: %d returning data_read = %d\n", fd, data_read);
+  
+  /* return total amount of bytes read */
+  return data_read; 
 }   
 
 
 
 
 /*
- * function: w_write (wrapper_write)
- *  purpose: Write "n" bytes to a descriptor.
- *   return: bytes written
+ * function: buf_write
+ *  purpose: prepare buffer that needs to be written
+ *   return: 
  */
-int
-w_write(unsigned int volatile fd, void *ptr, int volatile nbytes)
+void 
+buf_write(unsigned int fd, const char *ptr, size_t nbytes)
 {
+  memcpy(buf[fd], (void *) ptr, nbytes);
 
-  /* catch write timeouts */
-  if (sigsetjmp (sjmp, 1))
-  { 
-    logmessage("warning: write(): write timeout on fd: %d\n", fd);
-    
-    return (-1); /* close connection */
-  }
-
-  /* set the SIGALRM handler */
-  signal (SIGALRM, (void *) sigalrm_handler);
-  
-  /* read timeout */
-  alarm(3);
-  
-  /* write to a file descriptor */
-  if (f_write(fd, ptr, nbytes) != nbytes)
-  {
-    if(DEBUG > 2)
-      logmessage("DEBUG: fd: %d write(): failed, closing connection\n", fd);
-    
-    /* reset the timer & unset the signal handler */
-    alarm (0);           signal (SIGALRM, SIG_DFL);
-    return (-1);
-  } else {
-    if(DEBUG > 2)
-      logmessage("DEBUG: fd: %d write(%d) bytes\n", fd, nbytes);
-    
-    /* reset the timer & unset the signal handler */
-    alarm (0);           signal (SIGALRM, SIG_DFL);
-    return (nbytes);
-  }
+  buf_counter[fd] = 0; 
+  buf_size[fd] = nbytes;
 }
 
 
 
 
 /*
- * function: f_write (function_write)
- *  purpose: write "n" bytes to a descriptor.
+ * function: w_write (function_write)
+ *  purpose: write buffer to a descriptor.
  *   return: bytes written
  */
 ssize_t
-f_write(unsigned int fd, const void *vptr, size_t n)
+w_write(unsigned int fd, const void *vbuf)
 {
-  size_t     nleft;
-  ssize_t    nwritten;
-  const char *ptr;
+  ssize_t nbytes;
 
-  ptr = vptr;
-  nleft = n;
-  while (nleft > 0)
+  /* send data. disable signals are do not wait */
+  nbytes = send(fd, vbuf + buf_counter[fd], buf_size[fd] - buf_counter[fd], MSG_DONTWAIT | MSG_NOSIGNAL);
+
+  /* check if send returned an error */ 
+  if (nbytes == -1)
   {
-    if ((nwritten = write(fd, ptr, nleft)) <= 0)
+    /* ignore EAGAIN */
+    if (errno == EAGAIN) 
     {
-      if (nwritten < 0 && errno == EINTR)
-        nwritten = 0;           /* and call write() again */
-      else
-        return(-1);                     /* error */
+      if(DEBUG > 2)
+        logmessage("DEBUG: fd: %d write(): must try again after %d bytes (EAGAIN)\n", fd, nbytes);
+      
+      return nbytes;
     }
+    
+    /* something bad happened */
+    if (DEBUG > 2)
+      logmessage("DEBUG: fd: %d connection problem during write: %s\n", fd, strerror(errno));
 
-    nleft -= nwritten;
-    ptr   += nwritten;
+    return -1;
   }
-  return(n);
+
+  /* EOF was reached if n == 0 */
+  if (nbytes == 0)
+  {
+    if (DEBUG > 2)
+      logmessage("DEBUG: fd: %d connection got an EOF during write after %d bytes\n", fd, nbytes);
+     
+    return -1;
+  }
+
+  /* increment written buffer counter */
+  buf_counter[fd] += nbytes;
+
+  if (DEBUG > 2)
+    logmessage("DEBUG: fd: %d returning after %d bytes of data written\n", fd, nbytes);
+
+  /* check if buffer has been written */
+  if (buf_counter[fd] == buf_size[fd])
+    return -2;
+  
+  return nbytes;
 }
 
 
@@ -268,11 +295,13 @@ w_close(unsigned int fd)
   if(DEBUG > 2)
     logmessage("DEBUG: fd: %d shutting down fd %d\n", fd, fd);
 
+  /* shut down connection */
+  if (shutdown(fd, SHUT_RDWR) == -1)
+    logmessage("shutdown(%d): %s\n", fd, strerror(errno));
+  
   /* close a file descriptor */
   if (close(fd) == -1)
-  {
     logmessage("close(%d): %s\n", fd, strerror(errno));
-  }
 }   
 
 
@@ -285,6 +314,7 @@ w_close(unsigned int fd)
 int                     
 w_tcp_conn_acl (const char *host)
 {
+
   char            *p, range[strlen(CONN_ACL)];
   unsigned long   ip = inet_addr (host);          
     signed int    xi;
@@ -295,13 +325,16 @@ w_tcp_conn_acl (const char *host)
     p = range + xi;                         /* cycle backwards */
     if((range[xi] == ' ') || (range[xi] == ',') || (range == p))
     {                                       /* delimiter */
-      if(range == p)  p = range + xi;
-      else            p = range + xi + 1;
+      if(range == p)
+        p = range + xi;
+      else
+        p = range + xi + 1;
       if((p[strlen(p)-1] == ' ') || (p[strlen(p)-1] == ','))
         p[strlen(p)-1] = 0x00;              /* chomp trailing char */
 
-      if(cidr_ip_match(ip, p) == 1) return (1); /* pass */
-      *p = 0x00;                      /* chomp old match */
+      if(cidr_ip_match(ip, p) == 1) 
+        return (1);                         /* pass */
+      *p = 0x00;                            /* chomp old match */
     }
   }
 
